package org.petuum.app.nodedegree;

import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.petuum.jbosen.PsApplication;
import org.petuum.jbosen.PsTableGroup;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class NodeDegree extends PsApplication {
	private static final Logger logger = LoggerFactory
			.getLogger(NodeDegree.class);

	private NodeDegreeConfig config;
	private NodeDegreeWorker.Config mfConfig;
	private static final int NodeTableId = 0;

	public NodeDegree(NodeDegreeConfig config) {
		this.config = config;
		this.mfConfig = new NodeDegreeWorker.Config();
		// configure this.mfConfig for all workers
		this.mfConfig.staleness = config.staleness;
		
		this.mfConfig.N = config.N;
		this.mfConfig.M = config.M;
		this.mfConfig.dataFile = config.dataFile;
		this.mfConfig.NodeTableId = NodeTableId;
	}

	@Override
	public void initialize() {
		PsTableGroup.createDenseIntTable(NodeTableId, config.staleness, 1);
	}

	@Override
	public void runWorkerThread(int threadId) {
		int numClients = PsTableGroup.getNumClients();
		int numThreads = PsTableGroup.getNumLocalWorkerThreads();
		mfConfig.numWorkers = numClients * numThreads;
		
		int workerRank = numThreads * config.clientId + threadId;

		NodeDegreeWorker worker = new NodeDegreeWorker(mfConfig, workerRank);
		worker.run();
	}

	public static void main(String[] args) {
		final NodeDegreeConfig config = new NodeDegreeConfig();
		final CmdLineParser parser = new CmdLineParser(config);
		try {
			parser.parseArgument(args);
		} catch (CmdLineException e) {
			logger.error(e.getMessage());
			parser.printUsage(System.err);
			return;
		}
		long trainTimeBegin = System.currentTimeMillis();

		NodeDegree mfThreads = new NodeDegree(config);
		mfThreads.run(config);

		if (config.clientId == 0) {
			long trainTimeElapsed = System.currentTimeMillis() - trainTimeBegin;
			logger.info("The program took " + trainTimeElapsed + " ms.");
		}
	}
}


