package org.petuum.app.nodedegree;

import java.io.BufferedReader;
import java.io.FileReader;

import org.petuum.jbosen.PsTableGroup;
import org.petuum.jbosen.table.IntTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class NodeDegreeWorker implements Runnable {
    private static final Logger logger =
        LoggerFactory.getLogger(NodeDegreeWorker.class);

    private int workerRank;
    
    private String dataFile = "";
    private int N;
    private int edgeBegin;
    private int edgeEnd;
    
    private int NodeTableId;

    public static class Config {
        public int numWorkers = -1;
        public int staleness = 0;
        
        public String dataFile = "";
        public int N = 0;
        public int M = 0;
        
        public int NodeTableId = 0;
    }

    public NodeDegreeWorker(Config config, int workerRank) {
        this.workerRank = workerRank;
        
        int numEdgePerWorker = config.M / config.numWorkers;
        this.edgeBegin = this.workerRank * numEdgePerWorker;
        this.edgeEnd = (this.workerRank == config.numWorkers - 1) ? config.M 
        		: ( this.edgeBegin + numEdgePerWorker );
        
        this.N = config.N;
        this.dataFile = config.dataFile;
    }

    // Main worker thread logic.
    @Override
    public void run() {

    	IntTable nodetable = PsTableGroup.getIntTable(NodeTableId);
    	
        logger.info("Worker " + workerRank + " processes edges in ["
        		+ edgeBegin + ", " + edgeEnd + ")");
        
        try (BufferedReader br = new BufferedReader( new FileReader(dataFile) )) {
            String currLine;
            int edgeid = 0;
            while ((currLine = br.readLine()) != null) {
            	if (edgeid < edgeBegin ) continue;
            	if (edgeid >= edgeEnd ) break;
            	
                String[] fields = currLine.split(" ");
                int v1 = Integer.parseInt(fields[0]);
                
                nodetable.inc(v1, 0, 1);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

        PsTableGroup.globalBarrier();   //sync all workers
        
        if ( workerRank == 0 ) {
			String logstr = "Degree:";
			for ( int i=0; i<N; i++ ) {
				logstr += "\n" + "Node " + i + ": " + nodetable.get(i, 0);
			}
			logger.info(logstr);
        }
    }
}

